from db_hammer.mysql import MySQLConnection
from db_hammer.oracle import OracleConnection

from gencode.sqls import SQL


def DbConnect():
    from gencode.config import DB_CONF
    if DB_CONF["db_type"].upper() == "MYSQL":
        return MySQLConnection(**DB_CONF)
    elif DB_CONF["db_type"].upper() == "ORACLE":
        return OracleConnection(**DB_CONF)
    else:
        raise Exception("数据库类型不支持")


def get_all_tables():
    from gencode.config import DB_CONF
    with DbConnect() as db:
        return db.select_dict_list(sql=SQL["TABLES"][DB_CONF["db_type"].upper()], params={
            "db_name": DB_CONF["db_name"]
        })


# 取表备注
def get_table_comment(table_name: str):
    from gencode.config import DB_CONF
    sql = """
        Select TABLE_COMMENT AS comment from INFORMATION_SCHEMA.TABLES 
        Where table_schema = '{}' and  table_name='{}'
        """.format(DB_CONF["db_name"], table_name)

    with DbConnect() as db:
        obj = db.select_value(sql=sql)
        return obj.replace("主表", "").replace("从表", "").replace("明细表", "").replace("关联表", "")


def get_columns_by_table(table):
    from gencode.config import DB_CONF
    with DbConnect() as db:
        ls = db.select_dict_list(sql=SQL["COLUMNS"][DB_CONF["db_type"].upper()], params={
            "db_name": DB_CONF["db_name"],
            "table_name": table
        })
        for l in ls:
            l["comment"] = l["comment"].replace("\n\t", " ").replace("\n", " ").replace("\r", " ")
            if l["comment"] == "":
                l["comment"] = l["column_name"]
        return ls



SQL = {
    "TABLES":
        {
            "MYSQL": """
                        Select table_name,TABLE_COMMENT AS comment from INFORMATION_SCHEMA.TABLES 
                        Where table_schema = :db_name
                     """,
            "ORACLE": """
                    select table_name,comment from user_tab_comments order by table_name 
                    """
        },
    "COLUMNS": {
        "MYSQL": """
                       SELECT
                           COLUMN_NAME as column_name,
                            DATA_TYPE as data_type,
                            IFNULL(column_Comment,COLUMN_NAME) as comment,
                            ifnull(character_maximum_length,concat(numeric_precision,',',NUMERIC_SCALE)) as data_length
                        
                        
                         from INFORMATION_SCHEMA.COLUMNS t
                         Where table_name = :table_name AND 
                         table_schema = :db_name
                        AND (`COLUMN_KEY` != 'PRI')
                     """,
        "ORACLE": """
                    select
                     tc.column_name,
                     data_type,
                     cc.comments as comment,
                     data_length,
                     data_precision,
                     
                     
                     tc.data_default
                     from  user_tab_columns tc left join user_col_comments cc 
                     on tc.TABLE_NAME=cc.table_name and tc.COLUMN_NAME=cc.column_name
                     where tc.TABLE_NAME=:table_name
                    """
    },

}



import os

import yaml

DB_CONF = {
    "db_type": "MySQL",
    "db_name": "mysql",
    "user": "root",
    "pwd": "123456",
    "host": "127.0.0.1",
    "table_name": ["sys_menu_router"]
}

data_type_mapping = {
    "varchar(*)|text": "String"
}

gen_mode = 1

target_encoding = ""
jinja2_config = {}
template_path = ""
myself = {}


def load_conf(path):
    if not os.path.isfile(path):
        pwd = os.path.dirname(__file__)
        if os.path.isfile(os.path.join(pwd, path)):
            path = os.path.join(pwd, path)
        else:
            print(f"找不到配置文件：{path}")
            return False

    global DB_CONF, data_type_mapping, gen_mode, target_encoding, template_path, myself
    c = yaml.safe_load(open(path).read())
    template_path = c.get("templatePath", os.path.dirname(path))
    DB_CONF = c.get("dbConf", DB_CONF)
    data_type_mapping = c["dataTypeMapping"]
    data_type_mapping = c["dataTypeMapping"]
    gen_mode = c["genMode"]
    target_encoding = c.get("target", {}).get("encoding", "utf-8")
    jinja2_config["variable_start_string"] = c.get("variable_start_string", "{{")
    jinja2_config["variable_end_string"] = c.get("variable_end_string", "}}")
    myself = c.get("myself", "{}")
    return True



class Table:
    comment: str
    xx_table_name: str
    XX_TABLE_NAME: str
    XxTableName: str
    xxTableName: str
    xxtablename: str
    XXTABLENAME: str
    table_name: str
    TABLE_NAME: str
    tablename: str
    TABLENAME: str
    tableName: str
    TableName: str

    columns: []


class Column:
    comment: str
    column_name: str
    Column_Name: str
    COLUMN_NAME: str
    ColumnName: str
    columnname: str
    COLUMNNAME: str
    dataType: str
    data_length: int


class Gcode:
    is_dir = False
    temp_path: str
    temp_content: str
    target_path: str
    target_content: str
    encode: str



import os
import re

from db_hammer.util.file import get_dir_files

from gencode.config import jinja2_config
from gencode.models import Gcode


def get_XxTableName(xx_table_name):
    ts = str(xx_table_name).lower().split("_")
    s = ''
    for t in ts:
        if s == '':
            s = t.capitalize()
        else:
            s += t.capitalize()
    return s


def get_xxTableName(xx_table_name):
    ts = str(xx_table_name).lower().split("_")
    s = ''
    for t in ts:
        if s == '':
            s = t
        else:
            s += t.capitalize()
    return s


def get_table_name(xx_table_name):
    ind = str(xx_table_name).find("_")
    return str(xx_table_name[ind + 1:]).lower()


def get_table_package(xx_table_name):
    ind = str(xx_table_name).find("_")
    return str(xx_table_name[0:ind]).lower()


def get_tableName(xx_table_name):
    table_name = get_table_name(xx_table_name)
    ts = str(table_name).lower().split("_")
    s = ''
    for t in ts:
        if s == '':
            s = t
        else:
            s += t.capitalize()
    return s


def get_TableName(xx_table_name):
    table_name = get_table_name(xx_table_name)
    ts = str(table_name).lower().split("_")
    s = ''
    for t in ts:
        s += t.capitalize()
    return s


def get_columnName(coulum_name):
    ts = str(coulum_name).lower().split("_")
    s = ''
    for t in ts:
        if s == '':
            s = t
        else:
            s += t.capitalize()
    return s


def get_ColumnName(coulum_name):
    ts = str(coulum_name).lower().split("_")
    s = ''
    for t in ts:
        s += t.capitalize()
    return s


def get_path_list(path, encode="utf-8") -> [Gcode]:
    if path is None or path == "":
        path = "./"
    path = os.path.dirname(path)
    ll = get_dir_files(path=path, absolute=True, mode="AFD")
    ls = []
    for a in ll:
        if re.match(".*gcode$", a) or os.path.isdir(a):
            g = Gcode()
            g.temp_path = a
            g.encode = encode
            if os.path.isdir(a) and jinja2_config["variable_start_string"] in a and jinja2_config[
                "variable_end_string"] in a:
                g.is_dir = True
                g.target_path = a
                ls.append(g)
            if os.path.isfile(a):
                g.is_dir = False
                g.target_path = re.sub("\.gcode$", "", a)
                f = open(g.temp_path, 'rb')
                g.temp_content = f.read().decode(encode)
                f.close()
                ls.append(g)

    return ls


def get_dataType(db_data_type, data_length):
    from gencode.config import data_type_mapping
    for d_key in data_type_mapping.keys():
        d_keys = d_key.split("|")
        if db_data_type in d_keys or f"{db_data_type}({data_length})":
            return data_type_mapping[d_key]
    return db_data_type



import os

import click
from jinja2 import Template

from gencode.data import get_data
from gencode.models import Gcode
from gencode.utils import get_path_list


def render(data):
    from gencode.config import target_encoding, template_path
    # 路径经过模板
    for gcode in get_path_list(path=template_path, encode=target_encoding):
        g = to_template(gcode, data)
        if g is not None:
            gen_result(g)


def to_template(gcode: Gcode, data) -> Gcode:
    from gencode.config import jinja2_config
    try:
        if not gcode.is_dir:
            template = Template(gcode.temp_content, variable_start_string=jinja2_config["variable_start_string"],
                                variable_end_string=jinja2_config["variable_end_string"])
            txt = template.render(data)
            gcode.target_content = txt
        template = Template(gcode.target_path, variable_start_string=jinja2_config["variable_start_string"],
                            variable_end_string=jinja2_config["variable_end_string"])
        result = template.render(data)
        gcode.target_path = result
        return gcode
    except Exception as e:
        print("============错误==============")
        print(f"模板：{gcode.temp_path}")
        print(f"信息：{e}")


def gen_result(gcode: Gcode):
    """生成目录和文件"""
    if gcode.is_dir:
        os.makedirs(gcode.target_path, exist_ok=True)
    else:
        os.makedirs(os.path.dirname(gcode.target_path), exist_ok=True)
        f = open(gcode.target_path, mode="wb+")
        f.write(gcode.target_content.encode(encoding=gcode.encode))
        f.flush()
        f.close()


@click.command()
@click.option("--c", default="./config.yml", help="配置文件")
@click.option("--ui", default=0, help="启动UI的端口")
def run(c, ui):
    from gencode.config import load_conf
    if load_conf(c):
        d = get_data(table_name="sys_menu_router")
        render(data=d)


if __name__ == '__main__':
    run()



from setuptools import setup

with open("./README.md", "r",encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='genCode',
    version='0.0.3',
    description='生成代码',
    author='hammer',
    author_email='liuzhuogood@foxmail.com',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=['gencode'],
    package_data={'gencode': ['README.md', 'LICENSE']},
    install_requires=["pydantic", 'db-hammer', 'PyMySQL', 'cx_Oracle'],
    entry_points={
        'console_scripts': [
            'gencode = gencode.main:run'
        ]
    },
)



from db_hammer.util.date import date_to_str

from gencode.db import get_all_tables, get_columns_by_table
from gencode.models import Table, Column
from gencode.utils import *


def get_data(table_name=None):
    from gencode.config import myself
    objs = get_all_tables()
    ts = []
    c_table = Table()
    for obj in objs:
        table = Table()
        table.comment = obj["comment"]
        table.XX_TABLE_NAME = str(obj["table_name"]).upper()
        table.xx_table_name = str(obj["table_name"]).lower()
        table.XxTableName = get_XxTableName(obj["table_name"])
        table.xxTableName = get_xxTableName(obj["table_name"])
        table.xxtablename = table.xxTableName.lower()
        table.XXTABLENAME = table.xxTableName.upper()
        table.XX = get_table_package(obj["table_name"]).upper()
        table.xx = get_table_package(obj["table_name"]).lower()

        table.table_name = get_table_name(obj["table_name"])
        table.TABLE_NAME = str(table.table_name).upper()
        table.tableName = get_TableName(obj["table_name"]).lower()
        table.tableName = get_tableName(obj["table_name"])
        table.TableName = get_TableName(obj["table_name"])
        table.columns = []

        if table_name is not None:
            cols = get_columns_by_table(obj["table_name"])
            for c in cols:
                column = Column()
                column.comment = c["comment"]
                column.dataType = get_dataType(c["data_type"], c["data_length"])
                column.data_length = c["data_length"]
                column.column_name = str(c["column_name"]).lower()
                column.COLUMN_NAME = str(c["column_name"]).upper()
                column.ColumnName = get_ColumnName(c["column_name"])
                column.columnName = get_columnName(c["column_name"])
                column.columnname = get_columnName(c["column_name"]).lower()
                column.COLUMNNAME = get_columnName(c["column_name"]).upper()
                table.columns.append(column)
            c_table = table
        ts.append(table)

    d = {
        "tables": ts,
        "date": date_to_str(format_str="%Y-%m-%d"),
        "datetime": date_to_str(),
        "author": myself.get("author", ""),
        "mail": myself.get("mail", ""),
        "sign": myself.get("sign", ""),

    }
    d.update(c_table.__dict__)

    return d
